@@ -1,5 +1,5 @@
-"""Read OpenFoam Files for Python
-===========================================
+"""Reading OpenFoam Files with Python
+=====================================
 This module provides functions to read OpenFoam Files:
 
 .. autofunction:: readmesh
@@ -24,6 +24,14 @@ import gzip
 import struct
 import numpy as np
 
+# define color
+W = '\033[0m'  # white (normal)
+R = '\033[31m'  # red
+G = '\033[32m'  # green
+O = '\033[33m'  # orange
+B = '\033[34m'  # blue
+P = '\033[35m'  # purple
+
 
 def _make_path(path, time_name=None, name=None):
     if time_name is not None and name is None:  # pragma: no cover
@@ -65,13 +73,16 @@ class OpenFoamFile(object):
         order="F",
         precision=15,
         datatype=None,
+        verbose=True,
     ):
 
         self.pathcase = path
-        if time_name is "latestTime":
+        if time_name == "latestTime":
             time_name = _find_latesttime(path)
         self.path = _make_path(path, time_name, name)
-        print("Reading file " + self.path)
+        self.verbose = verbose
+        if self.verbose:
+            print("Reading file " + self.path)
 
         if not os.path.exists(self.path) and os.path.exists(self.path + ".gz"):
             self.path += ".gz"
@@ -106,20 +117,26 @@ class OpenFoamFile(object):
 
         self.boundary = self._parse_session(b"boundaryField")
 
-        if name is "boundary":
+        if name is None:
+            self._parse_data(boundary=boundary,
+                             precision=precision,
+                             datatype=datatype)
+        elif name.endswith("boundary"):
             self.boundaryface = self._parse_boundaryfile()
-        elif name is "faces":
+        elif name.endswith("faces"):
             self._parse_face()
-        elif name is "points":
+        elif name.endswith("points"):
             self._parse_points(precision=precision)
-        elif (name is "owner") or (name is "neighbour"):
+        elif name.endswith("owner") or name.endswith("neighbour"):
             self._parse_owner()
         else:
-            self._parse_data(
-                boundary=boundary, precision=precision, datatype=datatype
-            )
+            self._parse_data(boundary=boundary,
+                             precision=precision,
+                             datatype=datatype)
         if structured:
-            self._determine_order(order=order, precision=precision)
+            self._determine_order(boundary=boundary,
+                                  order=order,
+                                  precision=precision)
 
     def _parse_boundaryfile(self):
 
@@ -189,8 +206,9 @@ class OpenFoamFile(object):
             if b"value" in self.content.split(boun)[1].split(b"}")[0]:
                 data = self.content.split(boun)[1].split(b"value")[1]
             else:
-                print("Warning : No data on patch")
-                print("Nearest cells values use")
+                if self.verbose:
+                    print(R+"Warning : No data on boundary/patch")
+                    print("Using the values of the nearest cells"+W)
                 self._nearest_data(boundary=boundary, precision=precision)
                 return
         else:
@@ -229,14 +247,16 @@ class OpenFoamFile(object):
 
         if self.uniform:
             nb_pts = 1
-            if not (self.type_data is "scalar"):
+            if not (self.type_data == "scalar"):
                 data = shortline.split(b"(")[1]
                 data = data.replace(b" ", b"\n")
                 data = data.replace(b");", b"\n);")
             else:
                 data = words[1].split(b";")[0]
-            print("Warning : uniform field  of type " + self.type_data + "!\n")
-            print("Only constant field in output\n")
+            if self.verbose:
+                print(R+"Warning : uniform field  of type "
+                        + self.type_data + "!\n")
+                print("Only constant field in output\n"+W)
         elif shortline.count(b";") >= 1:
             nb_pts = int(shortline.split(b"(")[0])
             data = shortline.split(b"(")[1]
@@ -244,7 +264,9 @@ class OpenFoamFile(object):
             data = data.replace(b");", b"\n);")
         elif self.codestream:
             nb_pts = 0
-            print("Warning : codeStream field! I can not read the source code!\n")
+            if self.verbose:
+                print(R+"Warning : codeStream field! "
+                        + "I can not read the source code!\n"+W)
         else:
             nb_pts = int(lines[1])
             data = b"\n(".join(data.split(b"\n(")[1:])
@@ -284,10 +306,14 @@ class OpenFoamFile(object):
     def _nearest_data(self, boundary, precision):
 
         bounfile = OpenFoamFile(
-            self.pathcase + "/constant/polyMesh/", name="boundary"
+            self.pathcase + "/constant/polyMesh/",
+            name="boundary",
+            verbose=self.verbose
         )
         ownerfile = OpenFoamFile(
-            self.pathcase + "/constant/polyMesh/", name="owner"
+            self.pathcase + "/constant/polyMesh/",
+            name="owner",
+            verbose=self.verbose
         )
         id0 = int(bounfile.boundaryface[str.encode(boundary)][b"startFace"])
         nfaces = int(bounfile.boundaryface[str.encode(boundary)][b"nFaces"])
@@ -318,14 +344,16 @@ class OpenFoamFile(object):
 
         if self.uniform:
             nb_pts = 1
-            if not (self.type_data is "scalar"):
+            if not (self.type_data == "scalar"):
                 data = shortline.split(b"(")[1]
                 data = data.replace(b" ", b"\n")
                 data = data.replace(b");", b"\n);")
             else:
                 data = words[1].split(b";")[0]
-            print("Warning : uniform field  of type " + self.type_data + "!\n")
-            print("Only constant field in output\n")
+            if self.verbose:
+                print(R+"Warning : uniform field  of type "
+                        + self.type_data + "!\n")
+                print("Only constant field in output\n"+W)
         elif shortline.count(b";") >= 1:
             nb_pts = int(shortline.split(b"(")[0])
             data = shortline.split(b"(")[1]
@@ -333,7 +361,9 @@ class OpenFoamFile(object):
             data = data.replace(b");", b"\n);")
         elif self.codestream:
             nb_pts = 0
-            print("Warning : codeStream field! I can not read the source code!\n")
+            if self.verbose:
+                print(R+"Warning : codeStream field! "
+                        + "I can not read the source code!\n"+W)
         else:
             nb_pts = int(lines[1])
             data = b"\n(".join(data.split(b"\n(")[1:])
@@ -377,8 +407,8 @@ class OpenFoamFile(object):
                 nv = 9
             valuesbou = np.empty(nv * nfaces, dtype=float)
             for i in range(nfaces):
-                valuesbou[i * nv : (i + 1) * nv] = values[
-                    nv * cell[i] : nv * (cell[i] + 1)
+                valuesbou[i * nv: (i + 1) * nv] = values[
+                    nv * cell[i]: nv * (cell[i] + 1)
                 ]
             self.values = valuesbou
         if self.type_data == "vector":
@@ -410,22 +440,22 @@ class OpenFoamFile(object):
             nb_numbers = self.nfaces + 1
             self.pointsbyface = struct.unpack(
                 "{}i".format(nb_numbers),
-                data[0 : nb_numbers * struct.calcsize("i")],
+                data[0: nb_numbers * struct.calcsize("i")],
             )
-            data = self.content.split(str.encode(str(self.pointsbyface[-1])))[1]
+            data = self.content.split(
+                str.encode(str(self.pointsbyface[-1])))[1]
             data = b"\n(".join(data.split(b"\n(")[1:])
 
             for i in range(self.nfaces):
                 self.faces[i] = {}
-                self.faces[i]["npts"] = (
-                    self.pointsbyface[i + 1] - self.pointsbyface[i]
-                )
+                self.faces[i]["npts"] = self.pointsbyface[i + 1] - \
+                    self.pointsbyface[i]
                 self.faces[i]["id_pts"] = np.array(
                     struct.unpack(
                         "{}i".format(self.faces[i]["npts"]),
                         data[
                             self.pointsbyface[i]
-                            * struct.calcsize("i") : self.pointsbyface[i + 1]
+                            * struct.calcsize("i"): self.pointsbyface[i + 1]
                             * struct.calcsize("i")
                         ],
                     )
@@ -438,11 +468,9 @@ class OpenFoamFile(object):
                     break
                 else:
                     self.faces[i - 1] = {}
-                    self.faces[i - 1]["npts"] = line.split(b"(")[0]
-                    self.faces[i - 1]["id_pts"] = [
-                        int(s)
-                        for s in ((line.split(b"(")[1].split(b")")[0]).split())
-                    ]
+                    self.faces[i - 1]["npts"] = int(line.split(b"(")[0])
+                    self.faces[i - 1]["id_pts"] = [int(s) for s in
+                                                   (line.split(b"(")[1].split(b")")[0]).split()]
 
     def _parse_points(self, precision):
 
@@ -488,7 +516,7 @@ class OpenFoamFile(object):
                 continue
             break
         self.nb_faces = int(line)
-        data = self.content.split(line, 2)[2]
+        data = self.content.split(line, 2)[-1]
 
         self.type_data = self.header[b"class"]
 
@@ -508,9 +536,14 @@ class OpenFoamFile(object):
             self.values = np.array([int(s) for s in data.split()])
         self.nb_cell = np.max(self.values) + 1
 
-    def _determine_order(self, order, precision):
+    def _determine_order(self, boundary, order, precision):
 
-        xs, ys, zs = readmesh(self.pathcase, precision=precision)
+        xs, ys, zs = readmesh(
+            self.pathcase,
+            boundary=boundary,
+            precision=precision,
+            verbose=self.verbose
+        )
         nb_cell = xs.size
         nx = np.unique(xs).size
         ny = np.unique(ys).size
@@ -525,7 +558,7 @@ class OpenFoamFile(object):
         self.shape = (nx, ny, nz)
 
 
-def typefield(path, time_name=None, name=None):
+def typefield(path, time_name=None, name=None, verbose=True):
     """Read OpenFoam field and returns type of field.
 
     Args:
@@ -544,7 +577,7 @@ def typefield(path, time_name=None, name=None):
 
     path = _make_path(path, time_name, name)
 
-    field = OpenFoamFile(path)
+    field = OpenFoamFile(path, verbose=verbose)
 
     return field.type_data
 
@@ -558,6 +591,7 @@ def readfield(
     order="F",
     precision=15,
     datatype=None,
+    verbose=True,
 ):
     """
     Read OpenFoam field and reshape if necessary (structured mesh) and
@@ -570,11 +604,10 @@ def readfield(
         structured: False or True\n
         boundary: None or str\n
         order: "F" (default) or "C" \n
-        precision : Number of decimal places to round to (default: 15).
-        If decimals is negative, it specifies the number of positions to the
-        left of the decimal point. \n
+        precision : Number of decimal places to round to (default: 15)\n
         datatype: None (default) or str ("scalar", "vector"...) necessary in
-        case of files without header
+        case of files without header\n
+        verbose : True or False (default: True).
 
     Returns:
         array: array of type of the field; size of the array is the size of the
@@ -594,6 +627,7 @@ def readfield(
         order=order,
         precision=precision,
         datatype=datatype,
+        verbose=verbose,
     )
     values = field.values
 
@@ -634,6 +668,7 @@ def readscalar(
     order="F",
     precision=15,
     mode=None,
+    verbose=True,
 ):
     """
     Read OpenFoam scalar field and reshape if necessary and possible (not
@@ -646,9 +681,8 @@ def readscalar(
         structured: False or True\n
         boundary: None or str\n
         order: "F" (default) or "C" \n
-        precision : Number of decimal places to round to (default: 15).
-        If decimals is negative, it specifies the number of positions to the
-        left of the decimal point.
+        precision : Number of decimal places to round to (default: 15)\n
+        verbose : True or False (default: True).
 
     Returns:
         array: array of scalar field; size of the array is the size of the
@@ -670,6 +704,7 @@ def readscalar(
             order=order,
             precision=precision,
             datatype="scalar",
+            verbose=verbose,
         )
         values = scalar.values
 
@@ -690,6 +725,7 @@ def readvector(
     boundary=None,
     order="F",
     precision=15,
+    verbose=True,
 ):
     """
     Read OpenFoam vector field and reshape if necessary and possible (not
@@ -702,9 +738,8 @@ def readvector(
         structured: False or True\n
         boundary: None or str\n
         order: "F" (default) or "C" \n
-        precision : Number of decimal places to round to (default: 15).
-        If decimals is negative, it specifies the number of positions to the
-        left of the decimal point.
+        precision : Number of decimal places to round to (default: 15)\n
+        verbose : True or False (default: True).
 
     Returns:
         array: array of vector field; size of the array is the size of the
@@ -724,6 +759,7 @@ def readvector(
         order=order,
         precision=precision,
         datatype="vector",
+        verbose=verbose,
     )
     values = vector.values
 
@@ -733,7 +769,7 @@ def readvector(
     shape = (3, values.size // 3)
     values = np.reshape(values, shape, order=order)
     if structured:
-        if vector.uniform:
+        if vector.uniform and verbose:
             print("internalfield is uniform; so no reshape possible...")
         else:
             values[0:3, :] = values[0:3, vector.ind]
@@ -751,6 +787,7 @@ def readsymmtensor(
     boundary=None,
     order="F",
     precision=15,
+    verbose=True,
 ):
     """
     Read OpenFoam symmetrical tensor field and reshape if necessary and
@@ -763,9 +800,8 @@ def readsymmtensor(
         structured: False or True\n
         boundary: None or str\n
         order: "F" (default) or "C" \n
-        precision : Number of decimal places to round to (default: 15).
-        If decimals is negative, it specifies the number of positions to the
-        left of the decimal point.
+        precision : Number of decimal places to round to (default: 15)\n
+        verbose : True or False (default: True).
 
     Returns:
         array: array of symmetrical tensor field; size of the array is the size
@@ -785,6 +821,7 @@ def readsymmtensor(
         order=order,
         precision=precision,
         datatype="symmtensor",
+        verbose=verbose,
     )
     values = scalar.values
 
@@ -794,7 +831,7 @@ def readsymmtensor(
     shape = (6, values.size // 6)
     values = np.reshape(values, shape, order=order)
     if structured:
-        if scalar.uniform:
+        if scalar.uniform and verbose:
             print("internalfield is uniform; so no reshape possible...")
         else:
             values[0:6, :] = values[0:6, scalar.ind]
@@ -812,6 +849,7 @@ def readtensor(
     boundary=None,
     order="F",
     precision=15,
+    verbose=True,
 ):
     """
     Read OpenFoam tensor field and reshape if necessary and possible
@@ -824,9 +862,8 @@ def readtensor(
         structured: False or True\n
         boundary: None or str\n
         order: "F" (default) or "C" \n
-        precision : Number of decimal places to round to (default: 15).
-        If decimals is negative, it specifies the number of positions to the
-        left of the decimal point.
+        precision : Number of decimal places to round to (default: 15)\n
+        verbose : True or False (default: True).
 
     Returns:
         array: array of tensor field; size of the array is the size of the
@@ -846,6 +883,7 @@ def readtensor(
         order=order,
         precision=precision,
         datatype="tensor",
+        verbose=verbose,
     )
 
     values = scalar.values
@@ -856,7 +894,7 @@ def readtensor(
     shape = (9, values.size // 9)
     values = np.reshape(values, shape, order=order)
     if structured:
-        if scalar.uniform:
+        if scalar.uniform and verbose:
             print("internalfield is uniform; so no reshape possible...")
         else:
             values[0:9, :] = values[0:9, scalar.ind]
@@ -866,18 +904,26 @@ def readtensor(
     return values
 
 
-def readmesh(rep, structured=False, boundary=None, order="F", precision=15):
+def readmesh(
+    path,
+    time_name=None,
+    structured=False,
+    boundary=None,
+    order="F",
+    precision=15,
+    verbose=True
+):
     """
     Read OpenFoam mesh and reshape if necessary (in cartesian structured mesh).
 
     Args:
-        rep: str\n
+        path: str\n
+        time_name: str ('latestTime' is supported)\n
         structured: False or True\n
         boundary: None or str\n
         order: "F" (default) or "C" \n
-        precision : Number of decimal places to round to (default: 15).
-        If decimals is negative, it specifies the number of positions to the
-        left of the decimal point.
+        precision : Number of decimal places to round to (default: 15)\n
+        verbose : True or False (default: True).
 
     Returns:
         array: array of vector (Mesh X, Y, Z); size of the array is the size of
@@ -892,33 +938,57 @@ def readmesh(rep, structured=False, boundary=None, order="F", precision=15):
         X, Y, Z = fluidfoam.readmesh('path_of_OpenFoam_case', structured=True)
         So X, Y and Z are 3D numpy array with shape = (nx, ny, nz)
 
+    And if you play with dynamic mesh the time_name option is for you
+
     """
 
-    # backward compatibility (when ccx/ccy/ccz need)!!
-    if not os.path.exists(os.path.join(rep, "constant/polyMesh")):
-        rep = rep + "/.."
-
-    if not os.path.exists(os.path.join(rep, "constant/polyMesh")):
+    # hack
+    # because in dynamic Mesh cases, no polyMesh directory for initial time
+    if time_name == "0":
+        time_name = None
+    if not os.path.exists(os.path.join(path, "constant/polyMesh")):
         raise ValueError(
             "No constant/polyMesh directory in ",
-            rep,
+            path,
             " Please verify the directory of your case.",
         )
 
     if boundary is not None:
-        facefile = OpenFoamFile(rep + "/constant/polyMesh/", name="faces")
+        facefile = OpenFoamFile(
+            path + "/constant/polyMesh/", name="faces", verbose=verbose
+        )
         pointfile = OpenFoamFile(
-            rep + "/constant/polyMesh/", name="points", precision=precision
+            path + "/constant/polyMesh/",
+            name="faces",
+            verbose=verbose)
+        if time_name is not None:
+            pointfile = OpenFoamFile(
+                path=path,
+                time_name=time_name,
+                name="polyMesh/points",
+                precision=precision,
+                verbose=verbose
+            )
+        else:
+            pointfile = OpenFoamFile(
+                path + "/constant/polyMesh/",
+                name="points",
+                precision=precision,
+                verbose=verbose
+            )
+        bounfile = OpenFoamFile(
+            path + "/constant/polyMesh/",
+            name="boundary",
+            verbose=verbose
         )
-        bounfile = OpenFoamFile(rep + "/constant/polyMesh/", name="boundary")
         id0 = int(bounfile.boundaryface[str.encode(boundary)][b"startFace"])
-        nfaces = int(bounfile.boundaryface[str.encode(boundary)][b"nFaces"])
+        nmesh = int(bounfile.boundaryface[str.encode(boundary)][b"nFaces"])
 
-        xs = np.empty(nfaces, dtype=float)
-        ys = np.empty(nfaces, dtype=float)
-        zs = np.empty(nfaces, dtype=float)
+        xs = np.empty(nmesh, dtype=float)
+        ys = np.empty(nmesh, dtype=float)
+        zs = np.empty(nmesh, dtype=float)
 
-        for i in range(nfaces):
+        for i in range(nmesh):
             npts = int(facefile.faces[id0 + i]["npts"])
             id_pts = np.zeros(npts, dtype=int)
             id_pts[0:npts] = facefile.faces[id0 + i]["id_pts"][0:npts]
@@ -926,15 +996,43 @@ def readmesh(rep, structured=False, boundary=None, order="F", precision=15):
             ys[i] = np.mean(pointfile.values_y[id_pts[0:npts]])
             zs[i] = np.mean(pointfile.values_z[id_pts[0:npts]])
     else:
-        owner = OpenFoamFile(rep + "/constant/polyMesh/", name="owner")
-        if os.path.exists(os.path.join(rep, "constant/C")):
-            xs, ys, zs = readvector(rep, "constant", "C", precision=precision)
+        owner = OpenFoamFile(
+            path + "/constant/polyMesh/", name="owner", verbose=verbose
+        )
+        nmesh = owner.nb_cell
+        if time_name is None and os.path.exists(os.path.join(path,
+                                                             "constant/C")):
+            xs, ys, zs = readvector(
+                path, "constant", "C", precision=precision, verbose=verbose
+            )
+        elif time_name is not None and os.path.exists(_make_path(path,
+                                                                 time_name,
+                                                                 "C")):
+            xs, ys, zs = readvector(
+                path, time_name, "C", precision=precision, verbose=verbose
+            )
         else:
-            facefile = OpenFoamFile(rep + "/constant/polyMesh/", name="faces")
-            pointfile = OpenFoamFile(
-                rep + "/constant/polyMesh/", name="points", precision=precision
+            facefile = OpenFoamFile(
+                path + "/constant/polyMesh/", name="faces", verbose=verbose
+            )
+            if time_name is not None:
+                pointfile = OpenFoamFile(
+                    path=path,
+                    time_name=time_name,
+                    name="polyMesh/points",
+                    precision=precision,
+                    verbose=verbose
+                )
+            else:
+                pointfile = OpenFoamFile(
+                    path + "/constant/polyMesh/",
+                    name="points",
+                    precision=precision,
+                    verbose=verbose
+                )
+            neigh = OpenFoamFile(
+                path + "/constant/polyMesh/", name="neighbour", verbose=verbose
             )
-            neigh = OpenFoamFile(rep + "/constant/polyMesh/", name="neighbour")
             xs = np.empty(owner.nb_cell, dtype=float)
             ys = np.empty(owner.nb_cell, dtype=float)
             zs = np.empty(owner.nb_cell, dtype=float)
@@ -957,38 +1055,40 @@ def readmesh(rep, structured=False, boundary=None, order="F", precision=15):
                 zs[i] = np.mean(
                     pointfile.values_z[np.unique(np.concatenate(face[i])[:])]
                 )
-        if structured:
-            nx = np.unique(xs).size
-            ny = np.unique(ys).size
-            nz = np.unique(zs).size
-            if nx * ny * nz != owner.nb_cell:
-                raise ValueError(
-                    "nx.ny.nz not equal to number of cells."
-                    "Are you sure that your mesh is cartesian?"
-                )
-            ind = np.lexsort((xs, ys, zs))
-            shape = (nx, ny, nz)
-            xs = xs[ind].reshape(shape, order=order)
-            ys = ys[ind].reshape(shape, order=order)
-            zs = zs[ind].reshape(shape, order=order)
+    if structured:
+        nx = np.unique(xs).size
+        ny = np.unique(ys).size
+        nz = np.unique(zs).size
+        if nx * ny * nz != nmesh:
+            raise ValueError(
+                "nx.ny.nz not equal to number of cells."
+                "Are you sure that your mesh is cartesian?"
+                "Maybe try to use precision option"
+                "For example : "
+                "fluidfoam.readmesh(case, True, precision=13)"
+            )
+        ind = np.lexsort((xs, ys, zs))
+        shape = (nx, ny, nz)
+        xs = xs[ind].reshape(shape, order=order)
+        ys = ys[ind].reshape(shape, order=order)
+        zs = zs[ind].reshape(shape, order=order)
 
     return xs, ys, zs
 
 
 if __name__ == "__main__":
 
-    dirs = ["0.ascii", "0.asciigz", "0.bin", "0.bingz"]
+    dirs = ["ascii", "asciigz", "bin", "bingz"]
 
     for d in dirs:
-        rep = os.path.join(os.path.dirname(__file__), "../output_samples")
+        rep = os.path.join(os.path.dirname(__file__), "../output_samples/", d)
 
-        values = readscalar(rep, d, "alpha")
+        values = readscalar(rep, "0", "alpha")
 
-        values = readsymmtensor(rep, d, "sigma")
+        values = readsymmtensor(rep, "0", "sigma")
 
-        values = readtensor(rep, d, "Taus")
+        values = readtensor(rep, "0", "Taus")
 
-        values = readvector(rep, d, "U")
+        values = readvector(rep, "0", "U")
 
-        path = os.path.join(rep, d)
-        xs, ys, zs = readmesh(path)
+        xs, ys, zs = readmesh(rep)
