@@ -7,6 +7,7 @@ This module provides functions to list and read OpenFoam PostProcessing Files:
 """
 import os
 import numpy as np
+from glob import glob
 
 
 def _find_latesttime(path):
@@ -23,38 +24,13 @@ def _find_latesttime(path):
     return time_list[-1]
 
 
-def varinforce():
-    """ return the var included in postProcessing force files."""
-
-    return [
-        "T",
-        "Fpx",
-        "Fpy",
-        "Fpz",
-        "Fvx",
-        "Fvy",
-        "Fvz",
-        "Fpox",
-        "Fpoy",
-        "Fpoz",
-        "Mpx",
-        "Mpy",
-        "Mpz",
-        "Mvx",
-        "Mvy",
-        "Mvz",
-        "Mpox",
-        "Mpoy",
-        "Mpoz",
-    ]
-
-
 def readforce(path, namepatch="forces", time_name="0", name="forces"):
     """read the data contained in the force file .
     create the forces variables in the Forcesfile object
 
     Args:
         path: str\n
+        namepatch: str\n
         time_name: str ('latestTime' and 'mergeTime' are supported)\n
         name: str
 
@@ -63,20 +39,17 @@ def readforce(path, namepatch="forces", time_name="0", name="forces"):
         time
 
     A way you might use me is:\n
-        force = readforce('path_of_OpenFoam_case', '0', 'forces')
+        force = readforce(path='path_of_OpenFoam_case', namepatch='forces',
+                          time_name='0', name='forces')
 
-    It will create and fill the force variables:\n
-        ['T','Fpx','Fpy','Fpz','Fvx','Fvy','Fvz'
-        ,'Fpox','Fpoy','Fpoz','Mpx','Mpy','Mpz'
-        ,'Mvx','Mvy','Mvz','Mpox','Mpoy','Mpoz']
     """
 
-    path_namepatch = os.path.join(path, "postProcessing", namepatch)
+    path_namepatch = glob(f'{path}/**/'+namepatch, recursive=True)[0]
     if time_name is "latestTime":
         time_name = _find_latesttime(path_namepatch)
     elif time_name is "mergeTime":
         time_list = []
-        dir_list = os.listdir(path + "/postProcessing/" + namepatch)
+        dir_list = os.listdir(path_namepatch)
         for directory in dir_list:
             try:
                 float(directory)
@@ -107,7 +80,7 @@ def readforce(path, namepatch="forces", time_name="0", name="forces"):
     for dummy, line in enumerate(data[:-1]):
         if "#".encode() in line:
             j += 1
-        elif "#".encode() not in line and header == True:
+        elif "#".encode() not in line and header is True:
             header = False
             line = line.replace(b"(", b"")
             line = line.replace(b")", b"")
@@ -142,12 +115,17 @@ def readprobes(path, probes_name="probes", time_name="0", name="U"):
 
     """
 
-    path_probes_name = os.path.join(path, "postProcessing", probes_name)
+    time_vect = None
+    tab = None
+    jj = 0
+    probes_loc = None
+
+    path_probes_name = glob(f'{path}/**/'+probes_name, recursive=True)[0]
     if time_name is "latestTime":
         time_name = _find_latesttime(path_probes_name)
     elif time_name is "mergeTime":
         time_list = []
-        dir_list = os.listdir(path + "/postProcessing/" + probes_name)
+        dir_list = os.listdir(path_probes_name)
         for directory in dir_list:
             try:
                 float(directory)
@@ -157,7 +135,8 @@ def readprobes(path, probes_name="probes", time_name="0", name="U"):
         time_list.sort(key=float)
         time_list = np.array(time_list)
         for timename in time_list:
-            time_vect, tab = readprobes(path, probes_name, timename, name)
+            probes_loc, time_vect, tab = readprobes(
+                path, probes_name, timename, name)
             if "tab_merge" in locals():
                 for jj in range(np.size(time_vect[:])):
                     if time_vect[jj] > timevect_merge[-1]:
@@ -172,14 +151,26 @@ def readprobes(path, probes_name="probes", time_name="0", name="U"):
             else:
                 timevect_merge = time_vect
                 tab_merge = tab
-        return timevect_merge, tab_merge
+        return probes_loc, timevect_merge, tab_merge
 
     with open(os.path.join(path_probes_name, time_name, name), "rb") as f:
         content = f.readlines()
     j = 0
+    for dummy, line in enumerate(content):
+        if "#".encode() in line:
+            j += 1
+        elif "#".encode() not in line:
+            break
+    n_probes = j-2
+    probes_loc = np.zeros([n_probes, 3], dtype=float)
+    j = 0
     header = True
     for dummy, line in enumerate(content):
         if "#".encode() in line:
+            if j<n_probes:
+                for k in range(3):
+                    probes_loc[j, k] = (line.split(
+                        b"(")[1].split(b")")[0].split()[k])
             j += 1
         elif "#".encode() not in line and header:
             header = False
@@ -196,7 +187,7 @@ def readprobes(path, probes_name="probes", time_name="0", name="U"):
             print(
                 "Reading file "
                 + os.path.join(
-                    path, "postProcessing", probes_name, time_name, name
+                    path_probes_name, time_name, name
                 )
             )
             print(
@@ -223,4 +214,14 @@ def readprobes(path, probes_name="probes", time_name="0", name="U"):
                 values = probedata.split()
                 for l, vect in enumerate(values):
                     tab[j, k, l] = np.array(vect, dtype=float)
-    return time_vect, tab
+    if time_vect is None:
+        time_vect = np.array([])
+    if tab is None:
+        tab = np.array([])
+    return probes_loc, time_vect, tab
+
+
+if __name__ == "__main__":
+
+    rep = "../output_samples/ascii"
+    probes_loc, time_vect, dummy = readprobes(rep, time_name="latestTime")
