@@ -15,10 +15,10 @@ class SimpleTestCase(unittest.TestCase):
         print("create force object")
 
     def test_read_probes(self):
-        time_vect, dummy = fluidfoam.readprobes(case)
+        probes_loc, time_vect, dummy = fluidfoam.readprobes(case)
         self.assertEqual(10, len(time_vect))
-        time_vect, dummy = fluidfoam.readprobes(case, time_name="latestTime")
+        probes_loc, time_vect, dummy = fluidfoam.readprobes(case, time_name="latestTime")
         self.assertEqual(6, len(time_vect))
-        time_vect, dummy = fluidfoam.readprobes(case, time_name="mergeTime")
+        probes_loc, time_vect, dummy = fluidfoam.readprobes(case, time_name="mergeTime")
         self.assertEqual(13, len(time_vect))
-        time_vect, dummy = fluidfoam.readprobes(case, time_name="0", name="p")
+        probes_loc, time_vect, dummy = fluidfoam.readprobes(case, time_name="0", name="p")
